from typing import Optional

from pydantic import UUID4

from galileo_core.constants.request_method import RequestMethod
from galileo_core.constants.routes import Routes
from galileo_core.helpers.logger import logger
from galileo_core.schemas.base_config import GalileoConfig
from galileo_core.schemas.core.collaboration_role import CollaboratorRole
from galileo_core.schemas.core.group_project import GroupProjectCollaboratorRequest, GroupProjectCollaboratorResponse


def share_project_with_group(
    project_id: UUID4,
    group_id: UUID4,
    role: CollaboratorRole = CollaboratorRole.viewer,
    config: Optional[GalileoConfig] = None,
) -> GroupProjectCollaboratorResponse:
    config = config or GalileoConfig.get()
    logger.debug(f"Sharing project {project_id} with group {group_id} with role {role}...")
    response_dict = config.api_client.request(
        RequestMethod.POST,
        Routes.project_groups.format(project_id=project_id),
        json=[GroupProjectCollaboratorRequest(group_id=group_id, role=role).model_dump(mode="json")],
    )
    group_collaborators = [GroupProjectCollaboratorResponse.model_validate(group) for group in response_dict]
    logger.debug(f"Shared project {project_id} with group {group_id} with role {role}.")
    return group_collaborators[0]


def unshare_project_with_group(
    project_id: UUID4,
    group_id: UUID4,
    config: Optional[GalileoConfig] = None,
) -> None:
    config = config or GalileoConfig.get()
    logger.debug(f"Removing access for group {group_id} from project {project_id}")
    config.api_client.request(
        RequestMethod.DELETE, Routes.project_group.format(project_id=project_id, group_id=group_id)
    )
    logger.debug(f"Access for group {group_id} removed from project {project_id}.")



from typing import Literal, Optional

from pydantic import Field
from pydantic.types import UUID4

from galileo_core.schemas.logging.step import BaseStep, StepType


class Session(BaseStep):
    type: Literal[StepType.session] = Field(
        default=StepType.session, description=BaseStep.model_fields["type"].description
    )
    previous_session_id: Optional[UUID4] = None



from typing import Optional

from pydantic import UUID4

from galileo_core.constants.request_method import RequestMethod
from galileo_core.constants.routes import Routes
from galileo_core.exceptions.http import GalileoHTTPException
from galileo_core.helpers.logger import logger
from galileo_core.schemas.base_config import GalileoConfig
from galileo_core.schemas.core.collaboration_role import CollaboratorRole
from galileo_core.schemas.core.collaborator import UserCollaboratorCreate
from galileo_core.schemas.core.integration.user_integration import (
    UserIntegrationCollaboratorResponse,
)


def share_integration_with_user(
    integration_id: UUID4,
    user_id: UUID4,
    config: Optional[GalileoConfig] = None,
) -> UserIntegrationCollaboratorResponse:
    config = config or GalileoConfig.get()
    logger.debug(f"Sharing integration {integration_id} with user {user_id} with role {CollaboratorRole.viewer}...")
    response_dict = config.api_client.request(
        RequestMethod.POST,
        Routes.integration_users.format(integration_id=integration_id),
        json=[UserCollaboratorCreate(user_id=user_id).model_dump(mode="json")],
    )
    user_shared = [UserIntegrationCollaboratorResponse.model_validate(user) for user in response_dict]
    logger.debug(f"Shared integration {integration_id} with user {user_id} with role {CollaboratorRole.viewer}.")

    try:
        return user_shared[0]
    except IndexError:
        raise GalileoHTTPException(
            message="Galileo API returned empty response.",
            status_code=422,
            response_text=f"Response body {user_shared}",
        )
